%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Pisition estimation using discrete Kalman filtering.  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%Jess Garca Estbanez%%%
%%%GPS/IMU integration for a Snake robot whith active wheels 2009%%%

%%%%%%%Summary%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Stage 1 -PROBLEM DATA DEFINITION-  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        %filt_speed : This variable will define the velocity of the estimation. If we increase that variable the filter will arrive soon to a acceptable estimation but the more speed in the filter is going to punish the system with a bigger perturbation.
        %Ts: Defines the sample period time. 
        %accelt:(1x12) vector that represents the real aceleration (for the simulation) each time segment .
        %pos0: Initial position
        %accel0: Initial acceleration
        %w1 : Noise in the position sensor. Gaussian distribution
        %w2 : Noise in the acceleration sensor. Gaussian distribution
        
        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Stage 2 -SIMULATION OF THE REAL POSITION, ACCELERATION AND VELOCITY-   %
%          - GENERATION OF THE ACCELERATION AND POSITION SENSOR SIGNALS-  %  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        %accel : Real acceleration.
        %vel : Real velocity.
        %pos : Real position.
        %pos_sen: Position sensor measurement vector.
        %accel_sen: Acceleration sensor measurement vector
        

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Stage 3 -Space state definition-  %  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        %x(k+1)= F(k) x(k) + v(k+1)
        %z(k+1)=H(k+1) x(k+1) + w(k+1)
        %v and w are gaussian noises and their covariance matix are Q and R
        
        %State vector: x=[position, velocity, acceleration]'
        
        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Stage 4 -Kalman filtering-  %  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %We will run kfloop.m
        %The given results will be:
            %pos_est:estimated possition
            %vel_est:estimated velocity
            %accel_est:estimated velocity
        
%%%%%%%%%%%%%%%%%%%%%%%
%  Stage 5 -Results-  %  
%%%%%%%%%%%%%%%%%%%%%%%

    %Results plots

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
clear all;
close all;
format long;
clc;

%<<<<<<<<<<<<<STAGE 1>>>>>>>>>>>>>%

        Ts=1/120;                 %We will have a sample frecuency of 120 Hz
        filt_speed=Ts/100;        %Filter speed selection

        %Velocity vector definition.Each value represents the values we have
        %created for the example.

        accelxt=2*[+0.0  +0.6  -0.1  -1.0  0 -0.0  +0.3  +1.0  0  -1  -0.5 +0.6];
        accelyt=2*[+0.0  +0.0  -0.2  -0.4  0 0.0  +0.3  +0.3  0.3  0.5  0.0 -0.4];

        
        posx0=0;                  %Initial condition for the position.
        velx0=0;                  %Initial condition for the velocity.
        accelx0= accelxt(1);       %Initial condition for the acceleration.
        
        
        
        posy0=0;                  %Initial condition for the position.
        vely0=0;                  %Initial condition for the velocity.
        accely0= accelyt(1);       %Initial condition for the acceleration.     

        w1=1;             %Position noise 
        w2=0.02;             %Acceleration noise 
%<<<<<<<<<<<<<END STAGE 1>>>>>>>>>>>>>% 


%<<<<<<<<<<<<<STAGE 2>>>>>>>>>>>>>%

        st=round(10/Ts);    %Number of samples per segment
        Ts=10/st;           %Sampling time after round the number of samples per segment.
        t=Ts:Ts:120;        %Time vector.

        Lt=length(t);                                            % Total number of samples.
        samples=1:1:st*12;                                       % Sample number vector.

        %Object velocity simulation
        accelx=[];
        accely=[];
        
        for i=1:length(accelxt);
            accelx=[accelx;accelxt(i)*ones(st,1)];
        end
        
        for i=1:length(accelyt);
            accely=[accely;accelyt(i)*ones(st,1)];
        end
        
        
        velx= velx0 + cumsum(accelx)*Ts ;     %velocity x axis obtained by the acceleration integration.
        posx= posx0 + cumsum(velx)*Ts;        %position x axis obtained by the velocity integration
        
        posx=posx';
        velx=velx';
        accelx=accelx';
        
        noise1=w1*randn(1,Lt);          % Gaussian position noise with variance w1^2
        noise2=w2*randn(1,Lt);          % Gaussian acceleration noise with variance w2^2
        
        vely= vely0 + cumsum(accely)*Ts ;     %velocity in the y axix obtained by the acceleration integration.
        posy= posy0 + cumsum(vely)*Ts;        %position in the y axis obtained by the velocity integration
        
        posy=posy';
        vely=vely';
        accely=accely';
        
        
        offsetx=input('Select the offset in the x inercial sensor \n' );
        offsety=input('Select the offset in the y inercial sensor\n');
        
        simulation=menu('GPS SIGNAL','GPS SIGNAL WITHOUT INTERRUPTION','GPS SIGNAL INTRERRUPTION BETWEEN 30 AND 50 SEC');
        
        
        switch simulation
        
            case 1
        
                pos_senx=posx+noise1+0.1;                       % Measured x position  
                accel_senx=accelx+noise2+offsetx;               % Measured  x acceleration
                noise1=w1*randn(1,Lt);                          % Gaussian position noise with variance w1^2
                noise2=w2*randn(1,Lt);
                pos_seny=posy+noise1;                           % Measured y position
                accel_seny=accely+noise2+offsety;               % Measured y acceleration  
                
                velydirec= vely0 + cumsum(accel_seny)*Ts ;      %velocity y axis obtained by the acceleration measuremen integration.
                posydirec= posy0 + cumsum(velydirec)*Ts;        %position y axis obtained by the velocity measurement integration
                
                velxdirec= velx0 + cumsum(accel_senx)*Ts ;      %velocity x axis obtained by the direct acceleration measured integration.
                posxdirec= posx0 + cumsum(velxdirec)*Ts;        %position x axis obtained by the direct velocity measured integration
        
            case 2
                
                
                pos_senx=posx+noise1;
                pos_senx(1,30/Ts:50/Ts)=0+noise1(1,30/Ts:50/Ts);                        % Measured x position  
                accel_senx=accelx+noise2+offsetx;                                       % Measured  x acceleration
                noise1=w1*randn(1,Lt);                                                  % Gaussian position noise with variance w1^2
                noise2=w2*randn(1,Lt);
                pos_seny=posy+noise1;                                                   % Measured y position
                pos_seny(1,30/Ts:50/Ts)=0+noise1(1,30/Ts:50/Ts);
                accel_seny=accely+noise2+offsety;
                velxdirec= velx0 + cumsum(accel_senx)*Ts ;                              %velocity x axis obtained by the direct acceleration measured integration.
                posxdirec= posx0 + cumsum(velxdirec)*Ts;                                %position x axis obtained by the direct velocity measured integration
                velydirec= vely0 + cumsum(accel_seny)*Ts ;      %velocity y axis obtained by the acceleration measuremen integration.
                posydirec= posy0 + cumsum(velydirec)*Ts;        %position y axis obtained by the velocity measurement integration
                
        end
        
        
        
        
        
       
        

 %<<<<<<<<<<<<<END STAGE 2>>>>>>>>>>>>>%  
 
 
 %<<<<<<<<<<<<<STAGE 3>>>>>>>>>>>>>%
 
        A=[0 0 1 0 0 0;0 0 0 1 0 0;0 0 0 0 1 0; 0 0 0 0 0 1; 0 0 0 0 0 0;0 0 0 0 0 0];
        B=[0 0 0 0 0 0;0 0 0 0 0 0; 0 0 0 0 0 0; 0 0 0 0 0 0; 0 0 0 0 0 0; 0 0 0 0 0 0];
        C=[1 0 0 0 0 0;0 1 0 0 0 0;0 0 0 0 1 0; 0 0 0 0 0 1];                                                                 %Continuous system definition
        D=[0 0 0 0 0 0; 0 0 0 0 0 0; 0 0 0 0 0 0; 0 0 0 0 0 0];
        
        sys=ss(A,B,C,D);
        sys=c2d(sys,Ts,'zoh');  % Discretization 
        [F,G,H,Kd]=ssdata(sys);
        
        %Sensor matrix covariance. The elements that arent in the diagonal
        %are thero because the noise is uncorrelated.
        R=[w1^2  0  0  0 ;0  w1^2  0  0;0  0  w2^2 0;0  0  0  w2^2];
        
        Q=[ Ts^5/20           0            Ts^4/8          0          Ts^3/6        0;
            0             Ts^5/20             0          Ts^4/8          0        Ts^3/6;
            Ts^4/8            0            Ts^3/3          0           Ts^2/2       0; 
            0              Ts^4/8             0          Ts^3/3          0       Ts^2/2;
            Ts^3/6            0            Ts^2/2          0             Ts         0;
            0              Ts^3/6             0         Ts^2/2           0          Ts];
          
        Q=filt_speed*Q/norm(Q);
 
 %<<<<<<<<<<<<<END STAGE 3>>>>>>>>>>>>>%  
 
  %<<<<<<<<<<<<<STAGE 4>>>>>>>>>>>>>%
  
  %Seting the estimated variables starting value
  
    pos_estx(1)=posx(1);
    vel_estx(1)=0;
    accel_estx(1)=accelx(1);
    
    pos_esty(1)=posy(1);
    vel_esty(1)=0;
    accel_esty(1)=accely(1);
    
    P(:,:,1)= Q;
    XKK= [pos_estx(1);pos_esty(1); vel_estx(1);vel_esty(1); accel_estx(1);accel_esty(1)];    %X(k|k) (estimated)
    PKK=P(:,:,1);                                   %P(k|k) initial value
    
    % Calling the Kallman  loop
    
        for  k=1:Lt-1

            ZK1=[pos_senx(k+1); pos_seny(k+1); accel_senx(k+1); accel_seny(k+1)];             %Z(k+1) measured at k+1
            %XK1K1=X(k+1|k+1) PK1K1=P(k+1|k+1) state covariance
            [XK1K1,PK1K1,KK1]=fkloop(XKK,PKK,ZK1,F,H,Q,R); 
            
            
            
             Ka(:,:,k+1)=KK1;                       %Ka represents the evolution of the kalman gain    
             pos_estx(k+1)=XK1K1(1);
             pos_esty(k+1)=XK1K1(2);                %Estimated position given for the Kalman filter for the next step
             vel_estx(k+1)=XK1K1(3);
             vel_esty(k+1)=XK1K1(4);                %Estimated velocity given for the Kalman filter for the next step     
             accel_estx(k+1)=XK1K1(5);
             accel_esty(k+1)=XK1K1(6);              %Estimated acceleration given for the Kalman filter for the next step
             P(:,:,k+1)=PK1K1;
        
        %t+1 will be t in the next step in the loop
              PKK=PK1K1;
              XKK=XK1K1;
        
        end
        
        
        err_posx=(posx(1,:)-pos_estx(1,:));
        RMSD_xpos=sqrt(sum(err_posx.^2)/length(err_posx))
        
        err_posy=(posy(1,:)-pos_esty(1,:));
        RMSD_ypos=sqrt(sum(err_posy.^2)/length(err_posy))
        
        err_velx=(velx(1,:)-vel_estx(1,:));
        RMSD_xvel=sqrt(sum(err_velx.^2)/length(err_velx))
        
        err_vely=(vely(1,:)-vel_esty(1,:));
        RMSD_yvel=sqrt(sum(err_vely.^2)/length(err_vely))
        
        %<<<<<<<<<<<<<END STAGE 4>>>>>>>>>>>>>%
 
 
 %<<<<<<<<<<<<<STAGE 5>>>>>>>>>>>>>%
 
 %%%Measurements plots%%%
        figure('Name','Sensor x axis measurements')
        subplot(2,1,1)
        plot(t,posx,t,pos_senx)
        ylabel('position x (m)')
        xlabel('time (s)')
        legend('Real  x position','x Position sensor measurement')
        title(strcat('sensor n1, sigma(noise)=',num2str(std(noise1)),' m')) 
        grid on

        subplot(2,1,2)
        plot(t,accelx,t,accel_senx)
        ylabel('acceleration x (m/s^2)')
        xlabel('time (s)')
        legend('Real x acceleration',' x Acceleration sensor measurement')
        title(strcat('sensor n2, sigma(noise)=',num2str(std(noise2)),' m/s^2')) 
        grid on

    
        figure('Name','Sensor y axis measurements')
        subplot(2,1,1)
        plot(t,posy,t,pos_seny)
        ylabel('position y (m)')
        xlabel('time (s)')
        legend('Real  y position','y Position sensor measurement')
        title(strcat('sensor n1, sigma(noise)=',num2str(std(noise1)),' m')) 
        grid on

        subplot(2,1,2)
        plot(t,accely,t,accel_seny)
        ylabel('acceleration y (m/s^2)')
        xlabel('time (s)')
        legend('Real y acceleration',' y Acceleration sensor measurement')
        title(strcat('sensor n2, sigma(noise)=',num2str(std(noise2)),' m/s^2')) 
        grid on
        
         figure('Name','Robot trajectory measurement vs real robot trajectory ')
        plot3(posx,posy,t,pos_senx,pos_seny,t)
        ylabel('position x (m)')
        xlabel('position y (m)')
        zlabel('tme (sec)')
        legend('Real position','Position sensor measurement') 
        grid on
        
        figure('Name','Direct integration from the inercial sensor mesurement ')
        subplot(2,1,1)
        plot(t,posx,t,posxdirec)
        ylabel('position (m)')
        xlabel('time (s)')
        legend('Real x position','x Position obtained by a direct integration of IMU signal')
        grid on
        
        subplot(2,1,2)
        plot(t,posy,t,posydirec)
        ylabel('position (m)')
        xlabel('time (s)')
        legend('Real y position','y Position obtained by a direct integration of IMU signal') 
        grid on
        
        figure('Name','Robot trajectory from a direct integration of the IMU signals vs real robot trajectory ')
        plot3(posx,posy,t,posxdirec,posydirec,t)
        ylabel('position x (m)')
        xlabel('position y (m)')
        zlabel('tme (sec)')
        legend('Real position','Position obtained by a direct integration from IMU sensor')
        grid on
        
        
        %%%Kalman gain evolution%%%
    
        figure('Name','Kallman gain evolution')
        subplot(6,4,1)
        K11(:,1,1)=Ka(1,1,:);
        plot(t,K11)
        title('K(1,1)')
        xlabel('time (s)')
        grid on
        
        
        subplot(6,4,2)
        K12(:,1,1)=Ka(1,2,:);
        plot(t,K12)
        title('K(1,2)')
        xlabel('time (s)')
        grid on
        
        
        subplot(6,4,3)
        K13(:,1,1)=Ka(1,3,:);
        plot(t,K13)
        title('K(1,3)')
        xlabel('time (s)')
        grid on
        
        subplot(6,4,4)
        K14(:,1,1)=Ka(1,4,:);
        plot(t,K14)
        title('K(1,4)')
        xlabel('time (s)')
        grid on
        
        
        subplot(6,4,5)
        K21(:,1,1)=Ka(2,1,:);
        plot(t,K21)
        title('K(2,1)')
        xlabel('time (s)')
        grid on
        
        
        subplot(6,4,6)
        K22(:,1,1)=Ka(2,2,:);
        plot(t,K22)
        title('K(2,2)')
        xlabel('time (s)')
        grid on
        
        
        subplot(6,4,7)
        K23(:,1,1)=Ka(2,3,:);
        plot(t,K23)
        title('K(2,3)')
        xlabel('time (s)')
        grid on
        
        subplot(6,4,8)
        K24(:,1,1)=Ka(2,4,:);
        plot(t,K24)
        title('K(2,4)')
        xlabel('time (s)')
        grid on
        
        subplot(6,4,9)
        K31(:,1,1)=Ka(3,1,:);
        plot(t,K31)
        title('K(3,1)')
        xlabel('time (s)')
        grid on
        
        subplot(6,4,10)
        K32(:,1,1)=Ka(3,2,:);
        plot(t,K32)
        title('K(3,2)')
        xlabel('time (s)')
        grid on
        
        
        subplot(6,4,11)
        K33(:,1,1)=Ka(3,3,:);
        plot(t,K33)
        title('K(3,3)')
        xlabel('time (s)')
        grid on
        
        subplot(6,4,12)
        K34(:,1,1)=Ka(3,4,:);
        plot(t,K34)
        title('K(3,4)')
        xlabel('time (s)')
        grid on
        
        subplot(6,4,13)
        K41(:,1,1)=Ka(4,1,:);
        plot(t,K41)
        title('K(4,1)')
        xlabel('time (s)')
        grid on
        
        subplot(6,4,14)
        K42(:,1,1)=Ka(4,2,:);
        plot(t,K42)
        title('K(4,2)')
        xlabel('time (s)')
        grid on
        
        subplot(6,4,15)
        K43(:,1,1)=Ka(4,3,:);
        plot(t,K43)
        title('K(4,3)')
        xlabel('time (s)')
        grid on
        
        subplot(6,4,16)
        K44(:,1,1)=Ka(4,4,:);
        plot(t,K44)
        title('K(4,4)')
        xlabel('time (s)')
        grid on
        
        subplot(6,4,17)
        K51(:,1,1)=Ka(5,1,:);
        plot(t,K51)
        title('K(5,1)')
        xlabel('time (s)')
        grid on
        
        subplot(6,4,18)
        K52(:,1,1)=Ka(5,2,:);
        plot(t,K52)
        title('K(5,2)')
        xlabel('time (s)')
        grid on
        
        subplot(6,4,19)
        K53(:,1,1)=Ka(5,3,:);
        plot(t,K23)
        title('K(5,3)')
        xlabel('time (s)')
        grid on
        
        subplot(6,4,20)
        K54(:,1,1)=Ka(5,4,:);
        plot(t,K54)
        title('K(5,4)')
        xlabel('time (s)')
        grid on
        
        subplot(6,4,21)
        K61(:,1,1)=Ka(6,1,:);
        plot(t,K61)
        title('K(6,1)')
        xlabel('time (s)')
        grid on
        
        subplot(6,4,22)
        K62(:,1,1)=Ka(6,2,:);
        plot(t,K62)
        title('K(6,2)')
        xlabel('time (s)')
        grid on
        
        subplot(6,4,23)
        K63(:,1,1)=Ka(6,3,:);
        plot(t,K63)
        title('K(6,3)')
        xlabel('time (s)')
        grid on
        
        subplot(6,4,24)
        K64(:,1,1)=Ka(6,4,:);
        plot(t,K64)
        title('K(6,4)')
        xlabel('time (s)')
        grid on
        
        
 %%%P evolution%%%%
 
%  figure('Name','P evolution')
% 
% 
%         subplot(3,3,1)
%         P11(:,1,1)=P(1,1,:);
%         plot(t,P11)
%         title('P(1,1)')
%         xlabel('time (s)')
%         grid on
% 
%         subplot(3,3,2)
%         P12(:,1,1)=P(1,2,:);
%         plot(t,P12)
%         title('P(1,2)')
%         xlabel('time (s)')
%         grid on
% 
%         subplot(3,3,3)
%         P13(:,1,1)=P(1,3,:);
%         plot(t,P13)
%         title('P(1,3)')
%         xlabel('time (s)')
%         grid on
% 
%         subplot(3,3,4)
%         P21(:,1,1)=P(2,1,:);
%         plot(t,P21)
%         title('P(2,1)')
%         xlabel('time (s)')
%         grid on
% 
%         subplot(3,3,5)
%         P22(:,1,1)=P(2,2,:);
%         plot(t,P22)
%         title('P(2,2)')
%         xlabel('time (s)')
%         grid on
% 
%         subplot(3,3,6)
%         P23(:,1,1)=P(2,3,:);
%         plot(t,P23)
%         title('P(2,3)')
%         xlabel('time (s)')
%         grid on
% 
%         subplot(3,3,7)
%         P31(:,1,1)=P(3,1,:);
%         plot(t,P31)
%         title('P(3,1)')
%         xlabel('time (s)')
%         grid on
% 
%         subplot(3,3,8)
%         P32(:,1,1)=P(3,2,:);
%         plot(t,P32)
%         title('P(3,2)')
%         xlabel('time (s)')
%         grid on
% 
%         subplot(3,3,9)
%         P33(:,1,1)=P(3,3,:);
%         plot(t,P33)
%         title('P(3,3)')
%         xlabel('time (s)')
%         grid on
        
%%%Estimated measurement vs real ones%%%

figure('Name','Robot real trajectory ')
        plot3(posx,posy,t)
        ylabel('position x (m)')
        xlabel('position y (m)')
        Zlabel('time  (sec)')
        legend('Real position')        
        grid on;


figure('Name','Real state vs estimated state in the x axis')

        subplot(3,1,1)
        plot(t,posx,t,pos_estx)
        ylabel('x position (m)')
        xlabel('time (s)')
        legend('Real x position','Estimated x position')
        title(strcat('Ts=',num2str(Ts),  '  sigma(sensor n1)=',num2str(std(noise1)), 'sigma(sensor n2)=',num2str(std(noise2))))
        grid on

        subplot(3,1,2)
        plot(t,velx,t,vel_estx)
        ylabel('x Velocity (m/s)')
        xlabel('time (s)')
        legend('Real x velocity','Estimated x velocity')
        grid on

        subplot(3,1,3)
        plot(t,accelx,t,accel_estx)
        ylabel('x Acceleration(m/s^2)')
        xlabel('time (s)')
        legend('Real x acceleration','Estimated x acceleration')
        grid on
        
        
        figure('Name','Real state vd estimated state y axis')

        subplot(3,1,1)
        plot(t,posy,t,pos_esty)
        ylabel('y position (m)')
        xlabel('time (s)')
        legend('Real y position','Estimated y position')
        title(strcat('Ts=',num2str(Ts),  '  sigma(sensor n1)=',num2str(std(noise1)), 'sigma(sensor n2)=',num2str(std(noise2))))
        grid on

        subplot(3,1,2)
        plot(t,vely,t,vel_esty)
        ylabel('y Velocity (m/s)')
        xlabel('time (s)')
        legend('Real y velocity','Estimated y velocity')
        grid on

        subplot(3,1,3)
        plot(t,accely,t,accel_esty)
        ylabel('y Acceleration(m/s^2)')
        xlabel('time (s)')
        legend('Real y acceleration','Estimated y acceleration')
        grid on
        
        figure('Name','Robot trajectory estimated vs real robot trajectory ')
        plot3(posx,posy,t,pos_estx,pos_esty,t)
        ylabel('position x (m)')
        xlabel('position y (m)')
        Zlabel('time  (sec)')
        legend('Real position','Estimated position ')
         
        grid on