%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Position estimation using discrete Kalman filtering.  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%Jess Garca Estbanez%%%
%%%GPS/IMU integration for a Snake robot whith active wheels 2009%%%


%%%%%%%Summary%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Stage 1 -PROBLEM DATA DEFINITION-  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        %filt_speed : This variable will define the velocity of the estimation. If we increase that variable the filter will arrive soon to a acceptable estimation but the more speed in the filter is going to punish the system with a bigger perturbation.
        %Ts: Defines the sample period time. 
        %accelt:(1x12) vector that represents the real aceleration (for the simulation) each time segment .
        %pos0: Initial position
        %accel0: Initial acceleration
        %w1 : Noise in the position sensor. Gaussian distribution
        %w2 : Noise in the acceleration sensor. Gaussian distribution
        
        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Stage 2 -SIMULATION OF THE REAL POSITION, ACCELERATION AND VELOCITY-   %
%          - GENERATION OF THE ACCELERATION AND POSITION SENSOR SIGNALS-  %  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        %accel : Real acceleration.
        %vel : Real velocity.
        %pos : Real position.
        %pos_sen: Position sensor measure vector.
        %accel_sen: Acceleration sensor measure vector
        

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Stage 3 -Space state definition-  %  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        %x(k+1)= F(k) x(k) + v(k+1)
        %z(k+1)=H(k+1) x(k+1) + w(k+1)
        %v and w are gaussian noises and their covariance matix are Q and R
        
        %State vector: x=[position, velocity, acceleration]'
        
        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Stage 4 -Kalman filtering-  %  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %We will run kfloop.m
        %The given results will be:
            %pos_est:estimated possition
            %vel_est:estimated velocity
            %accel_est:estimated velocity
        
%%%%%%%%%%%%%%%%%%%%%%%
%  Stage 5 -Results-  %  
%%%%%%%%%%%%%%%%%%%%%%%

    %Results plots

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
clear all;
close all;
format long;
clc;

%<<<<<<<<<<<<<STAGE 1>>>>>>>>>>>>>%

        Ts_pos=1/4;                     %We will have a sample the position frecuency of 4 Hz with the GPS
        Ts2=1/120;
        filt_speed=Ts_pos/100;          %Filter speed selection
        Ts_2=1/120;
        %Velocity vector definition.Each value represents the values we have
        %created for the example.

        accelxt=2*[+0.0  +0.6  -0.1  -1.0  0 -0.0  +0.3  +1.0  0  -1  -0.5 +0.6];
        accelyt=2*[+0.0  +0.0  -0.2  -0.4  0 0.0  +0.3  +0.3  0.3  0.5  0.0 -0.4];
        angle_velt=0.6*[0 0.04 0.05 0.07 0.04   0  0 0  0 -0.02 -0.02 -0.02]; 

        
        posx0=0;                    %Initial condition for the position.
        velx0=0;                    %Initial condition for the velocity.
        accelx0= accelxt(1);        %Initial condition for the acceleration.
        angle0=0;                   %Initial condition for the head angle.
        angle_vel0=angle_velt(1);
        
        posy0=0;                    %Initial condition for the position.
        vely0=0;                    %Initial condition for the velocity.
        accely0= accelyt(1);        %Initial condition for the acceleration.     

        w1=2.5;                     %Position noise. 
        w2=0.02;                    %Acceleration noise.
        w3=0.005;                    %Angle velocity error
%<<<<<<<<<<<<<END STAGE 1>>>>>>>>>>>>>% 


%<<<<<<<<<<<<<STAGE 2>>>>>>>>>>>>>%

        st2=round(10/Ts2);          %Number of samples per segment
        Ts2=10/st2;                 %Sampling time after round the number of samples per segment.
        t2=Ts2:Ts2:120;             %Time vector.

        Lt2=length(t2);             % Total number of samples.
        samples2=1:1:st2*12;        % Sample number vector.

        st_pos=round(10/Ts_pos);    %Number of samples per segment
        Ts_pos=10/st_pos;           %Sampling time after round the number of samples per segment.
        t_pos=Ts_pos:Ts_pos:120;    %Time vector.
        
        Lt_pos=length(t_pos);       % Total number of samples.
        samples_pos=1:1:st_pos*12;  % Sample number vector.

        
        
        
        
        %Object velocity simulation
        accelx=[];
        accely=[];
        angle_vel=[];
        for i=1:length(accelxt);
            accelx=[accelx;accelxt(i)*ones(st2,1)];
        end
        
        for i=1:length(accelyt);
            accely=[accely;accelyt(i)*ones(st2,1)];
        end
        
        for i=1:length(angle_velt);
            angle_vel=[angle_vel;angle_velt(i)*ones(st2,1)];
        end
        
        
        velx= velx0 + cumsum(accelx)*Ts2 ;      %velocity x axis obtained by the acceleration integration.
        posx= posx0 + cumsum(velx)*Ts2;         %position x axis obtained by the velocity integration
        angle=angle0+cumsum(angle_vel)*Ts2;     %angle obtained by the velocity integration.
        vely= vely0 + cumsum(accely)*Ts2 ;      %velocity in the y axix obtained by the acceleration integration.
        posy= posy0 + cumsum(vely)*Ts2;         %position in the y axis obtained by the velocity integration
       
        posx=posx';
        posy=posy';
        velx=velx';
        vely=vely';
        accelxrel=cos(angle).*accelx+sin(angle).*accely;
        accelyrel=-sin(angle).*accelx+cos(angle).*accely;
        
        accelyrel=accelyrel';
        accelxrel=accelxrel';
        angle_vel=angle_vel';
        angle=angle';
        noise1=w1*randn(1,Lt_pos);          % Gaussian position noise with variance w1^2
        noise2=w2*randn(1,Lt2);          % Gaussian acceleration noise with variance w2^2
        noise3=w3*randn(1,Lt2);
       
        
        
        
        
        
        
        
        
        
        posx_resample=downsample(posx,Ts_pos/Ts2);
        posy_resample=downsample(posy,Ts_pos/Ts2);
        
        offsetx=input('Select the offset in the x inercial sensor \n' );
                offsety=input('Select the offset in the y inercial sensor\n');
                offset3=input('Select the offset in the gyroscope\n');
        
        simulation=menu('GPS SIGNAL','GPS SIGNAL WITHOUT INTERRUPTION','GPS SIGNAL INTRERRUPTION BETWEEN 30 AND 50 SEC');
        
        
        switch simulation
        
            case 1
        
                
                
                
                pos_senx=posx_resample+noise1;                       % Measured x position  
                pos_seny=posy_resample+noise1;                       % Measured y position
                accel_senx=accelxrel+noise2+offsetx;                 % Measured  x acceleration
                accel_seny=accelyrel+noise2+offsety;                 % Measured y acceleration  
                accelx_resample=downsample(accelxrel,Ts_pos/Ts2);
                
                accely_resample=downsample(accelyrel,Ts_pos/Ts2);
                anglevel_sen=angle_vel+noise3+offset3;
                
                angle_velresample=downsample(anglevel_sen,Ts_pos/Ts2);
                
                angle_direct=angle0+cumsum(anglevel_sen)*Ts2;     %angle obtained by the angle measurement integration
                velxdirec= velx0 + cumsum(accel_senx.*cos(angle_direct)-accel_seny.*sin(angle_direct))*Ts2 ;      %velocity x axis obtained by the acceleration measuremen integration.
                velydirec= vely0 + cumsum(accel_senx.*sin(angle_direct)+accel_seny.*cos(angle_direct))*Ts2 ;      %velocity y axis obtained by the acceleration measuremen integration.
                posxdirec= posx0 + cumsum(velxdirec)*Ts2;        %position x axis obtained by the velocity measurement integration
                posydirec= posy0 + cumsum(velydirec)*Ts2;        %position y axis obtained by the velocity measurement integration
                
            
            
            case 2
                
                  pos_senx=posx_resample+noise1;                       % Measured x position  
                  pos_seny=posy_resample+noise1;                       % Measured y position  

                  pos_senx(1,30/Ts_pos:50/Ts_pos)=0+noise1(1,30/Ts_pos:50/Ts_pos);                       % Measured x position  
                  pos_seny(1,30/Ts_pos:50/Ts_pos)=0+noise1(1,30/Ts_pos:50/Ts_pos);
                  accel_senx=accelxrel+noise2+offsetx;                 % Measured  x acceleration
                  accel_seny=accelyrel+noise2+offsety;                 % Measured y acceleration  
                  
                  accelx_resample=downsample(accelxrel,Ts_pos/Ts2);
                  accely_resample=downsample(accelyrel,Ts_pos/Ts2);
                  
                  anglevel_sen=angle_vel+noise3+offset3;
                  angle_velresample=downsample(angle_vel,Ts_pos/Ts2);
                
                  angle_direct=angle0+cumsum(anglevel_sen)*Ts2;     %angle obtained by the angle measurement integration
                  velxdirec= velx0 + cumsum(accel_senx.*cos(angle_direct)-accel_seny.*sin(angle_direct))*Ts2 ;      %velocity x axis obtained by the acceleration measuremen integration.
                  velydirec= vely0 + cumsum(accel_senx.*sin(angle_direct)+accel_seny.*cos(angle_direct))*Ts2 ;      %velocity y axis obtained by the acceleration measuremen integration.
                  posxdirec= posx0 + cumsum(velydirec)*Ts2;        %position x axis obtained by the velocity measurement integration
                  posydirec= posy0 + cumsum(velydirec)*Ts2;        %position y axis obtained by the velocity measurement integration
                
        end
        
        
        
        
        
       
        

 %<<<<<<<<<<<<<END STAGE 2>>>>>>>>>>>>>%  
 
 
 %<<<<<<<<<<<<<STAGE 3>>>>>>>>>>>>>%
 
        A=[0 0 1 0 0 0 0 0;0 0 0 1 0 0 0 0;0 0 0 0 1 0 0 0;0 0 0 0 0 1 0 0;0 0 0 0 0 0 0 0;0 0 0 0 0 0 0 0;0 0 0 0 0 0 0 1;0 0 0 0 0 0 0 0];
        B=[0 0 0 0 0 0 0 0;0 0 0 0 0 0 0 0;0 0 0 0 0 0 0 0;0 0 0 0 0 0 0 0;0 0 0 0 0 0 0 0;0 0 0 0 0 0 0 0;0 0 0 0 0 0 0 0;0 0 0 0 0 0 0 0];
        C=[1 0 0 0 0 0 0 0;0 1 0 0 0 0 0 0;0 0 0 0 1 0 0 0;0 0 0 0 0 1 0 0;0 0 0 0 0 0 0 1];                                                                 %Continuous system definition
        D=[0 0 0 0 0 0 0 0;0 0 0 0 0 0 0 0;0 0 0 0 0 0 0 0;0 0 0 0 0 0 0 0;0 0 0 0 0 0 0 0];
        
        sys=ss(A,B,C,D);
        sys=c2d(sys,Ts_pos,'zoh');  % Discretization 
        [F,G,H,Kd]=ssdata(sys);
        
        %Sensor matrix covariance. The elements that arent in the diagonal
        %are thero because the noise is uncorrelated.
        R=[w1^2  0  0  0  0;0  w1^2  0  0 0;0  0  w2^2 0 0;0  0  0  w2^2 0;0 0 0 0 w3^2];
        
        Q=[ Ts_pos^5/20       0       Ts_pos^4/8          0         Ts_pos^3/6        0            0         0;
            0            Ts_pos^5/20       0         Ts_pos^4/8         0        Ts_pos^3/6        0         0;
          Ts_pos^4/8          0       Ts_pos^3/3          0         Ts_pos^2/2        0            0         0; 
            0            Ts_pos^4/8        0         Ts_pos^3/3         0        Ts_pos^2/2        0         0;
          Ts_pos^3/6          0        Ts_pos^2/2         0          Ts_pos           0            0         0;
            0            Ts_pos^3/6        0         Ts_pos^2/2         0          Ts_pos          0         0;
            0                 0            0              0             0             0        Ts_pos^3/3    Ts_pos^2/2;
            0                 0            0              0             0             0        Ts_pos^2/2      Ts_pos];
        
        
        Q=filt_speed*Q/norm(Q);
     
        
 %<<<<<<<<<<<<<END STAGE 3>>>>>>>>>>>>>%  
 
  %<<<<<<<<<<<<<STAGE 4>>>>>>>>>>>>>%
  
  %Seting the estimated variables starting value
  
    pos_estx(1)=pos_senx(1);
    vel_estx(1)=0;
    accel_estx(1)=accel_senx(1);
    angle_est(1)=0;
    anglevel_est(1)=anglevel_sen(1);
    pos_esty(1)=pos_seny(1);
    vel_esty(1)=0;
    accel_esty(1)=accel_seny(1);
    
    P(:,:,1)= Q;
    XKK= [pos_estx(1);pos_esty(1); vel_estx(1);vel_esty(1); accel_estx(1);accel_esty(1);angle_est(1);anglevel_est(1)];    %X(k|k) (estimated)
    PKK=P(:,:,1);                                   %P(k|k) initial value
    
    % Calling the Kallman  loop
    
        for  k=1:Lt_pos-1

            ZK1=[pos_senx(k+1); pos_seny(k+1); accelx_resample(k+1); accely_resample(k+1);angle_velresample(k+1)];             %Z(k+1) measured at k+1
            
            H=[1 0 0 0 0 0 0 0;0 1 0 0 0 0 0 0;0 0 0 0 cos(XKK(7)) sin(XKK(7)) -sin(XKK(7))*XKK(5)+cos(XKK(7))*XKK(6) 0;0 0 0 0 -sin(XKK(7)) cos(XKK(7)) -cos(XKK(7))*XKK(5)-sin(XKK(7))*XKK(6) 0;0 0 0 0 0 0 0 1];
            
            [XK1K1,PK1K1,KK1]=fkloop_extended(XKK,PKK,ZK1,F,H,Q,R); 
            
            
            
             Ka(:,:,k+1)=KK1;                       %Ka represents the evolution of the kalman gain    
             pos_estx(k+1)=XK1K1(1);
             pos_esty(k+1)=XK1K1(2);                %Estimated position given for the Kalman filter for the next step
             vel_estx(k+1)=XK1K1(3);
             vel_esty(k+1)=XK1K1(4);                %Estimated velocity given for the Kalman filter for the next step     
             accel_estx(k+1)=XK1K1(5);
             accel_esty(k+1)=XK1K1(6);              %Estimated acceleration given for the Kalman filter for the next step
             angle_est(k+1)=XK1K1(7);               %Estimated head angle given for the Kalman filter for the next step
             anglevel_est(k+1)=XK1K1(8);            %Estimated head angle velocity given for the Kalman filter for the next step
             
             P(:,:,k+1)=PK1K1;
        
        %t+1 will be t in the next step in the loop
              PKK=PK1K1;
              XKK=XK1K1;
        
        end
        
        
        err_posx=(posx_resample(1,:)-pos_estx(1,:));
        RMSD_xpos=sqrt(sum(err_posx.^2)/length(err_posx))
        
        err_posy=(posy_resample(1,:)-pos_esty(1,:));
        RMSD_ypos=sqrt(sum(err_posy.^2)/length(err_posy))
        
        err_posx_direct=(posx(1,:)-posxdirec(1,:));
        RMSD_xpos_direct=sqrt(sum(err_posx_direct.^2)/length(err_posx_direct))
        
        err_posy_direct=(posy(1,:)-posydirec(1,:));
        RMSD_ypos_direct=sqrt(sum(err_posy_direct.^2)/length(err_posy_direct))
        
        
        
        %<<<<<<<<<<<<<END STAGE 4>>>>>>>>>>>>>%
 
 
 %<<<<<<<<<<<<<STAGE 5>>>>>>>>>>>>>%
 
 %%%Measurements plots%%%
        figure('Name','Sensor x axis measurements')
        subplot(3,1,1)
        plot(t2,posx,t_pos,pos_senx)
        ylabel('position x (m)')
        xlabel('time (s)')
        legend('Real  x position','x Position sensor measurement')
        title(strcat('sensor n1, sigma(noise)=',num2str(std(noise1)),' m')) 
        grid on
        
        subplot(3,1,2)
        plot(t2,accelxrel,t2,accel_senx)
        ylabel('acceleration x (m/s^2)')
        xlabel('time (s)')
        legend('Real relative x acceleration','Realitive x acceleration sensor measurement')
        title(strcat('sensor n2, sigma(noise)=',num2str(std(noise2)),' m/s^2')) 
        grid on
        
        subplot(3,1,3)
        plot(t2,angle_vel,t2,anglevel_sen)
        ylabel('Angle  (rad)')
        xlabel('time (s)')
        legend('Real head angle velocity','Head angle velocity measurement')
        title(strcat('sensor n3, sigma(noise)=',num2str(std(noise3)),' rad/s')) 
        grid on

    
        figure('Name','Sensor y axis measurements')
        subplot(3,1,1)
        plot(t2,posy,t_pos,pos_seny)
        ylabel('position y (m)')
        xlabel('time (s)')
        legend('Real  y position','y Position sensor measurement')
        title(strcat('sensor n1, sigma(noise)=',num2str(std(noise1)),' m')) 
        grid on

        subplot(3,1,2)
        plot(t2,accelyrel,t2,accel_seny)
        ylabel('acceleration y (m/s^2)')
        xlabel('time (s)')
        legend('Real y acceleration',' y Acceleration sensor measurement')
        title(strcat('sensor n2, sigma(noise)=',num2str(std(noise2)),' m/s^2')) 
        grid on
        
        subplot(3,1,3)
        plot(t2,angle_vel,t2,anglevel_sen)
        ylabel('Angle  (rad)')
        xlabel('time (s)')
        legend('Real head angle velocity','Head angle velocity measurement')
        title(strcat('sensor n3, sigma(noise)=',num2str(std(noise3)),' rad/s')) 
        grid on
        
         figure('Name','Robot trajectory measurement vs real robot trajectory ')
        plot3(posx,posy,t2,pos_senx,pos_seny,t_pos)
        ylabel('position x (m)')
        xlabel('position y (m)')
        zlabel('tme (sec)')
        legend('Real position','Position sensor measurement') 
        grid on
        
        figure('Name','Direct integration from the inercial sensor mesurement ')
        subplot(3,1,1)
        plot(t2,posx,t2,posxdirec)
        ylabel('position (m)')
        xlabel('time (s)')
        legend('Real x position','x Position obtained by a direct  integration of IMU signal')
        grid on
        
        subplot(3,1,2)
        plot(t2,posy,t2,posydirec)
        ylabel('position (m)')
        xlabel('time (s)')
        legend('Real y position','y Position obtained by a direct integration of IMU signal') 
        grid on
        
        subplot(3,1,3)
        plot(t2,angle,t2,angle_direct)
        ylabel('angle (rad)')
        xlabel('time (s)')
        legend('Real angle','Angel obtained by a direct integration of gyroscope signal') 
        grid on

          figure('Name','Robot trajectory from a direct integration of the IMU signals vs real robot trajectory ')
        plot3(posx,posy,t2,posxdirec,posydirec,t2)
        ylabel('position x (m)')
        xlabel('position y (m)')
        zlabel('tme (sec/100)')
        legend('Real position','Position obtained by a direct integration from IMU sensor')
        grid on
        
        
        %%%Kalman gain evolution%%%
    
        figure('Name','Kallman gain evolution')
        subplot(8,5,1)
        K11(:,1,1)=Ka(1,1,:);
        plot(t_pos,K11)
        title('K(1,1)')
        xlabel('time (s)')
        grid on
        
        
        subplot(8,5,2)
        K12(:,1,1)=Ka(1,2,:);
        plot(t_pos,K12)
        title('K(1,2)')
        xlabel('time (s)')
        grid on
        
        
        subplot(8,5,3)
        K13(:,1,1)=Ka(1,3,:);
        plot(t_pos,K13)
        title('K(1,3)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,4)
        K14(:,1,1)=Ka(1,4,:);
        plot(t_pos,K14)
        title('K(1,4)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,5)
        K15(:,1,1)=Ka(1,5,:);
        plot(t_pos,K15)
        title('K(1,5)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,6)
        K21(:,1,1)=Ka(2,1,:);
        plot(t_pos,K21)
        title('K(2,1)')
        xlabel('time (s)')
        grid on
        
        
        subplot(8,5,7)
        K22(:,1,1)=Ka(2,2,:);
        plot(t_pos,K22)
        title('K(2,2)')
        xlabel('time (s)')
        grid on
        
        
        subplot(8,5,8)
        K23(:,1,1)=Ka(2,3,:);
        plot(t_pos,K23)
        title('K(2,3)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,9)
        K24(:,1,1)=Ka(2,4,:);
        plot(t_pos,K24)
        title('K(2,4)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,10)
        K25(:,1,1)=Ka(2,5,:);
        plot(t_pos,K25)
        title('K(2,5)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,11)
        K31(:,1,1)=Ka(3,1,:);
        plot(t_pos,K31)
        title('K(3,1)')
        xlabel('time (s)')
        grid on
        
        
        subplot(8,5,12)
        K32(:,1,1)=Ka(3,2,:);
        plot(t_pos,K32)
        title('K(3,2)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,13)
        K33(:,1,1)=Ka(3,3,:);
        plot(t_pos,K33)
        title('K(3,3)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,14)
        K34(:,1,1)=Ka(3,4,:);
        plot(t_pos,K34)
        title('K(3,4)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,15)
        K35(:,1,1)=Ka(3,5,:);
        plot(t_pos,K35)
        title('K(3,5)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,16)
        K41(:,1,1)=Ka(4,1,:);
        plot(t_pos,K41)
        title('K(4,1)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,17)
        K42(:,1,1)=Ka(4,2,:);
        plot(t_pos,K42)
        title('K(4,2)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,18)
        K43(:,1,1)=Ka(4,3,:);
        plot(t_pos,K43)
        title('K(4,3)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,19)
        K44(:,1,1)=Ka(4,4,:);
        plot(t_pos,K44)
        title('K(4,4)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,20)
        K45(:,1,1)=Ka(4,5,:);
        plot(t_pos,K45)
        title('K(4,5)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,21)
        K51(:,1,1)=Ka(5,1,:);
        plot(t_pos,K51)
        title('K(5,1)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,22)
        K52(:,1,1)=Ka(5,2,:);
        plot(t_pos,K52)
        title('K(5,2)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,23)
        K53(:,1,1)=Ka(5,3,:);
        plot(t_pos,K53)
        title('K(5,3)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,24)
        K54(:,1,1)=Ka(5,4,:);
        plot(t_pos,K54)
        title('K(5,4)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,25)
        K55(:,1,1)=Ka(5,5,:);
        plot(t_pos,K55)
        title('K(5,5)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,26)
        K61(:,1,1)=Ka(6,1,:);
        plot(t_pos,K61)
        title('K(6,1)')
        xlabel('time (s)')
        grid on

          subplot(8,5,27)
        K62(:,1,1)=Ka(6,2,:);
        plot(t_pos,K62)
        title('K(6,2)')
        xlabel('time (s)')
        grid on
        
          subplot(8,5,28)
        K63(:,1,1)=Ka(6,3,:);
        plot(t_pos,K63)
        title('K(6,3)')
        xlabel('time (s)')
        grid on
        
          subplot(8,5,29)
        K64(:,1,1)=Ka(6,4,:);
        plot(t_pos,K64)
        title('K(6,4)')
        xlabel('time (s)')
        grid on
        
          subplot(8,5,30)
        K65(:,1,1)=Ka(6,5,:);
        plot(t_pos,K65)
        title('K(6,5)')
        xlabel('time (s)')
        grid on
        
          subplot(8,5,31)
        K71(:,1,1)=Ka(7,1,:);
        plot(t_pos,K71)
        title('K(7,1)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,32)
        K72(:,1,1)=Ka(7,2,:);
        plot(t_pos,K72)
        title('K(7,2)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,33)
        K73(:,1,1)=Ka(7,3,:);
        plot(t_pos,K73)
        title('K(7,3)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,34)
        K74(:,1,1)=Ka(7,4,:);
        plot(t_pos,K74)
        title('K(7,4)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,35)
        K75(:,1,1)=Ka(7,5,:);
        plot(t_pos,K75)
        title('K(7,5)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,36)
        K81(:,1,1)=Ka(8,1,:);
        plot(t_pos,K81)
        title('K(8,1)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,37)
        K82(:,1,1)=Ka(8,2,:);
        plot(t_pos,K82)
        title('K(8,2)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,38)
        K83(:,1,1)=Ka(8,3,:);
        plot(t_pos,K83)
        title('K(8,3)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,39)
        K84(:,1,1)=Ka(8,4,:);
        plot(t_pos,K84)
        title('K(8,4)')
        xlabel('time (s)')
        grid on
        
        subplot(8,5,40)
        K85(:,1,1)=Ka(8,5,:);
        plot(t_pos,K85)
        title('K(8,5)')
        xlabel('time (s)')
        grid on

figure('Name','Robot real trajectory ')
        plot3(posx,posy,t2)
        ylabel('position x (m)')
        xlabel('position y (m)')
        Zlabel('time  (sec)')
        legend('Real position')        
        grid on;


figure('Name','Real state vs estimated state in the x axis')

        subplot(4,1,1)
        plot(t2,posx,t_pos,pos_estx)
        ylabel('x position (m)')
        xlabel('time (s)')
        legend('Real x position','Estimated x position')
        title(strcat('Ts=',num2str(Ts_pos),  '  sigma(sensor n1)=',num2str(std(noise1)), 'sigma(sensor n2)=',num2str(std(noise2)), 'sigma(sensor n3)=',num2str(std(noise3))))
        grid on

        subplot(4,1,2)
        plot(t2,velx,t_pos,vel_estx)
        ylabel('x Velocity (m/s)')
        xlabel('time (s)')
        legend('Real x velocity','Estimated x velocity')
        grid on

        subplot(4,1,3)
        plot(t2,accelx,t_pos,accel_estx)
        ylabel('x Acceleration(m/s^2)')
        xlabel('time (s)')
        legend('Real x acceleration','Estimated x acceleration')
        grid on
        
        subplot(4,1,4)
        plot(t2,angle,t_pos,angle_est)
        ylabel('Angle(rad)')
        xlabel('time (s)')
        legend('Real angle','Estimated angle')
        grid on
        
        
        figure('Name','Real state vd estimated state y axis')

        subplot(4,1,1)
        plot(t2,posy,t_pos,pos_esty)
        ylabel('y position (m)')
        xlabel('time (s)')
        legend('Real y position','Estimated y position')
        title(strcat('Ts=',num2str(Ts_pos),  '  sigma(sensor n1)=',num2str(std(noise1)), 'sigma(sensor n2)=',num2str(std(noise2)), 'sigma(sensor n3)=',num2str(std(noise3))))
        grid on

        subplot(4,1,2)
        plot(t2,vely,t_pos,vel_esty)
        ylabel('y Velocity (m/s)')
        xlabel('time (s)')
        legend('Real y velocity','Estimated y velocity')
        grid on

        subplot(4,1,3)
        plot(t2,accely,t_pos,accel_esty)
        ylabel('y Acceleration(m/s^2)')
        xlabel('time (s)')
        legend('Real y acceleration','Estimated y acceleration')
        grid on
        
        subplot(4,1,4)
        plot(t2,angle,t_pos,angle_est)
        ylabel('Angle(rad)')
        xlabel('time (s)')
        legend('Real angle','Estimated angle')
        grid on
        
        figure('Name','Robot trajectory estimated vs real robot trajectory ')
        plot3(posx,posy,t2,pos_estx,pos_esty,t_pos)
        ylabel('position x (m)')
        xlabel('position y (m)')
        Zlabel('time  (sec)')
        legend('Real position','Estimated position ')
        grid on
        
        